use bindings;
use core::ops::*;

const fn fourcc_code(a: u8, b: u8, c: u8, d: u8) -> u32 {
    (a as u32) | (b as u32) << 8 | (c as u32) << 16 | (d as u32) << 24
}

// TODO: Figure out a more automated way of importing this

#[derive(Copy, Clone, PartialEq, Eq)]
#[repr(u32)]
pub enum Format {
    XRGB888 = fourcc_code(b'X', b'R', b'2', b'4')
}

#[macro_export]
macro_rules! drm_format_list {
    ($($fmt:expr),+) => {
        [$($fmt as u32),+, 0]
    };
}

#[repr(transparent)]
#[derive(Copy, Clone)]
pub struct FormatInfo {
    inner: bindings::drm_format_info,
}

impl FormatInfo {
    // SAFETY: `ptr` must point to a valid instance of a `bindings::drm_format_info`
    pub(super) unsafe fn from_raw<'a>(ptr: *const bindings::drm_format_info) -> &'a Self {
        // SAFETY: Our data layout is identical
        unsafe { &*ptr.cast() }
    }

    /// The number of color planes (1 to 3)
    pub const fn num_planes(&self) -> u8 {
        self.inner.num_planes
    }

    /// Does the format embed an alpha component?
    pub const fn has_alpha(&self) -> bool {
        self.inner.has_alpha
    }

    /// The total number of components (color planes + alpha channel, if there is one)
    pub const fn num_components(&self) -> u8 {
        self.num_planes() + self.has_alpha() as u8
    }

    /// Number of bytes per block (per plane), where blocks are defined as a rectangle of pixels
    /// which are stored next to each other in a byte aligned memory region.
    pub fn char_per_block(&self) -> &[u8] {
        // SAFETY: The union we access here is just for descriptive purposes on the C side, both
        // members are identical in data layout
        unsafe { &self.inner.__bindgen_anon_1.char_per_block[..self.num_components() as _] }
    }
}

impl AsRef<bindings::drm_format_info> for FormatInfo {
    fn as_ref(&self) -> &bindings::drm_format_info {
        &self.inner
    }
}

impl From<bindings::drm_format_info> for FormatInfo {
    fn from(value: bindings::drm_format_info) -> Self {
        Self { inner: value }
    }
}
