// SPDX-License-Identifier: GPL-2.0 OR MIT

//! DRM core plane blending related functions

use bindings;
use core::ffi::c_uint;

/// Try to simplify a rotation.
///
/// Attempt to simplify the rotation to a form that is supported. Eg. if the hardware supports
/// everything except `REFLECT_X` one could call this function like this:
///
/// ```
/// rotation_simplify(rotation, ROTATE_0 | ROTATE_90 | ROTATE_180 | ROTATE_270 | REFLECT_Y)
/// ```
///
/// to eliminate the `REFLECT_X` flag. Depending on what kind of transforms the hardware supports,
/// this function may not be able to produce a supported transform, so the caller should check the
/// result afterwards.
#[inline]
#[must_use]
pub fn rotation_simplify(rotation: u32, supported_rotations: u32) -> c_uint {
    // SAFETY: FFI call with no special requirements
    unsafe { bindings::drm_rotation_simplify(rotation, supported_rotations) }
}
